/**
 * \file Config.cpp
 *
 *  \date Jan 31, 2012
 *  \author Alexandre Tuleu
 */

#include "Config.h"

#include <libatuleu/options/Parsers.h>

#include <libwebots/communication/client.hh>
#include <libwebots/plugin/modules/jointmanager/jointmanagerbuilder.hh>

namespace liboncilla {
namespace webots {
namespace config {
	SectionConfig::SectionConfig(options::Section::Ptr base)
		: d_section(base){}

	SectionConfig::~SectionConfig(){}
	options::Section::Ptr & SectionConfig::Section(){
		return d_section;
	}

	PassiveDynamicsSection::PassiveDynamicsSection(options::Section::Ptr base)
		: SectionConfig(base)
		, d_oStiffness(base->AddOption<double>("stiffness",0.0,"stiffness of teh spring"))
		, d_oPreCompression(base->AddOption<double>("precompression",0.0,"precompression of the spring in length/angle"))
		, d_oDamping(base->AddOption<double>("damping",0.0,"damping"))
		, d_oMaxLength(base->AddOption<double>("max-length",0.0,"maximal length of the spring"))
		, d_oMinLength(base->AddOption<double>("min-length",0.0,"mininaml length of the spring")){}

	PassiveDynamicsSection::~PassiveDynamicsSection(){}

	void PassiveDynamicsSection::SendJointDefinition(uint32_t legIdentifier) const {
		using namespace libwebots::communication;
		using namespace libwebots::plugin::modules::messages;

		Client & client = Client::Instance();

		client.Send(builders::JointManager::SetJointPassiveDynamic(legIdentifier,
		                                                           Stiffness(),
		                                                           Damping(),
		                                                           PreCompression()));
	}


	LegSection::LegSection(options::Section::Ptr base)
		: SectionConfig(base)
		, d_oDiagonalErp(base->AddOption<double>("erp",0.4,"erp of the diagonal spring"))
		, d_oDiagonalCfm(base->AddOption<double>("cfm",0.4,"erp of the diagonal spring"))
		, d_oMargin(base->AddOption("diagonal-margin",0.05,"margin for the diagonal joint to avoird instabilities"))
		, d_oMaxSpeed(base->AddOption("max-speed",1.0,"max speed of d1-d2 joint"))
		, d_diagonal(base->AddSubSection("diagonal-spring","diagonal spring system properties"))
		, d_parallel(base->AddSubSection("parallel-spring","propperty of the parallel spring if any"))
		, d_foot(base->AddSubSection("foot-spring","property of the foot spring")){}

	LegSection::~LegSection(){}

	const PassiveDynamicsSection & LegSection::Diagonal() const{ return d_diagonal; }
	const PassiveDynamicsSection & LegSection::Parallel() const{ return d_parallel; }
	const PassiveDynamicsSection & LegSection::Foot()     const{ return d_foot; }


	ContactSection::ContactSection(options::Section::Ptr parent)
		: SectionConfig(parent)
		, d_oBounce(parent->AddOption<double>("bounce",0.5,""))
		, d_oBounceVelocity(parent->AddOption<double>("bounce-velocity",0.01,""))
		, d_oCoulombFriction(parent->AddOption<double>("coulomb",1.0,""))
		, d_oFds(parent->AddOption<double>("fds",0.0,""))
		, d_oErp(parent->AddOption<double>("erp",0.2,""))
		, d_oCfm(parent->AddOption<double>("cfm",1e-5,""))
		, d_oDraw(parent->AddOption<bool>("draw",false,""))
		{}

	ContactSection::~ContactSection(){};


	MainSection::MainSection(options::Section::Ptr parent)
		: SectionConfig(parent)
		, d_oDebugLibwebots(parent->AddOption<bool>("debug-libwebots",false,"add additionnal debug output from libwebots internals"))
		, d_oInitTime(parent->AddOption<double>("init-time",1.5,"init time of the simulation"))
		, d_oProcessLoopTime(parent->AddOption("process-loop-time",0.006,"process loop desired time")){
	}

	MainSection::~MainSection(){}

	Config::Config()
		:	d_root(new options::Section("",""))
		, d_foreLeg(d_root->AddSubSection("fore-leg","fore leg configuration"))
		, d_hindLeg(d_root->AddSubSection("hind-leg","hind leg configuration"))
		, d_contact(d_root->AddSubSection("contact","contact configuration"))
		, d_main(d_root->AddSubSection("simulation","global simulation settings")){
	}

	Config::~Config(){}

	void Config::LoadConfigFile(){
		options::ConfigFileParser p;
		p.Parse("../../config/liboncilla-webots.config",d_root);
	}
	const LegSection & Config::ForeLeg(){
		return d_foreLeg;
	}
	const LegSection & Config::HindLeg(){
		return d_hindLeg;
	}
	const ContactSection & Config::Contact(){
		return d_contact;
	}

	const MainSection & Config::Main() {
		return d_main;
	}
}
} /* namespace liboncilla */
} /* namespace webots */

