/**
 * \file WebotsL2Adapter.cpp
 *
 *  \date Jan 12, 2012
 *  \author Alexandre Tuleu
 */

#include "WebotsL2Adapter.h"

#include <stdexcept>

#include <libwebots/communication/client.hh>
#include <libwebots/plugin/modules/jointmanager/jointmanagerbuilder.hh>

using namespace rci::oncilla;
using namespace libwebots::communication;
namespace lm=libwebots::plugin::modules::messages;


WebotsL2Adapter::WebotsL2Adapter(rci::oncilla::OncillaL2Ptr node,
             	                uint32_t identifier, webots::Servo * servo,
             	                const liboncilla::webots::config::LegSection & conf)
	: d_node(node)
	, d_servo(*servo)
	, d_jointIdentifier(identifier)
	, d_minPosition(conf.Diagonal().MinLength())
	, d_maxPosition(conf.Diagonal().MaxLength())
	, d_maxSpeed(std::abs(conf.MaxSpeed())){
	if(d_maxPosition <= d_minPosition){
		std::swap(d_minPosition,d_maxPosition);
	}
	if(!servo){
		throw std::logic_error("Unvalid webots::Servo handle.");
	}
	Init(conf);
}

WebotsL2Adapter::~WebotsL2Adapter(){
}




void WebotsL2Adapter::PreStep(double ts){
	rci::ControlModePtr c = d_node->getControlMode();
	if(c->supports(rci::jointComplianceControl) ||
	   c->supports(rci::jointVelocityControl) ||
	   c->supports(rci::jointTorqueControl)){
		throw std::runtime_error("L2 impedance, velocity and torque control are"
				" not currently supported in liboncilla-webots.");
	}

	if(c->supports(rci::jointPositionControl) ){
		using namespace lm::builders;
		//get the command and apply joint limit position
		double command = std::max(std::min(d_node->unsafeGetCommand(),1.0),0.0);
		command *= d_maxPosition - d_minPosition;
		command += d_minPosition;

		//now apply speed limitation
		double commandRate = command - d_lastCommand;
		commandRate /= ts;

		if(std::abs(commandRate) > d_maxSpeed){
			command = d_lastCommand + ts * d_maxSpeed * (commandRate > 0 ? 1.0 : -1.0);
			//still inside the position boundaries by construction
		}

		Client::Instance().Send(JointManager::SetJointParameter(d_jointIdentifier,lm::jointmanager::JointParameters::dParamLoStop,command));
		d_lastCommand = command;
	} else {
		throw std::runtime_error("Please use jointPositionControl for L2.");
	}
}

void WebotsL2Adapter::PostStep(double ts){
	d_node->unsafeUpdateMotorAxisPosition(d_servo.getPosition());
	//TODO : fixe the value when there is spring compression (issue #376)
	d_node->unsafeUpdateMagneticEncoderPosition(d_servo.getPosition());

}

void WebotsL2Adapter::Activate(){
	namespace c = liboncilla::webots::config;
	unsigned int ts = c::Config::Instance().Main().ProcessLoopTime() * 1000;
	d_servo.enableMotorForceFeedback(ts);
	d_servo.enablePosition(ts);

};

void WebotsL2Adapter::DeActivate(){
	d_servo.disableMotorForceFeedback();
	d_servo.disablePosition();
};


void WebotsL2Adapter::Init(const liboncilla::webots::config::LegSection & conf){

	Client & c = Client::Instance();

	using namespace lm::builders;


	JointManager::JointParameterValue * p;
	JointManager::ListOfParameterValue pv;
	pv.Reserve(4);

	p = pv.Add();
	double margin = conf.Margin() * (d_maxPosition - d_minPosition);
	p->set_parameter(lm::jointmanager::JointParameters::dParamHiStop);
	p->set_value(d_maxPosition + margin);


	p->set_parameter(lm::jointmanager::JointParameters::dParamLoStop);
	p->set_value(d_minPosition);

	p->set_parameter(lm::jointmanager::JointParameters::dParamStopERP);
	p->set_value(conf.DiagonalErp());

	p->set_parameter(lm::jointmanager::JointParameters::dParamStopERP);
	p->set_value(conf.DiagonalCfm());

	c.Send(JointManager::SetJointParameters(d_jointIdentifier,pv));

}
